﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SAP.Middleware.Connector;


namespace Excel2SapBom
{
    public class SapConnection
    {
        private static Dictionary<String, SapConnection> singletonConnections;
        private static SapLogonIniConfiguration iniConfiguration;

        private RfcDestination destination;
        private RfcCustomDestination customDestination;
        private RfcConfigParameters parameters;
        private List<RfcDestinationMonitor> destinationMonitors;

        static SapConnection()
        {
            SapConnection.iniConfiguration = SapLogonIniConfiguration.Create();
            RfcDestinationManager.RegisterDestinationConfiguration(SapConnection.iniConfiguration);
            SapConnection.singletonConnections = new Dictionary<string, SapConnection>();
        }

        public static String[] GetSystems
        {
            get
            {
                return SapConnection.iniConfiguration.GetEntries();
            }
        }

        public static SapConnection Factory(String destinationName)
        {
            SapConnection retValue;

            if (SapConnection.singletonConnections.ContainsKey(destinationName))
            {
                SapConnection.singletonConnections.TryGetValue(destinationName, out retValue);
            }
            else
            {
                retValue = new SapConnection(destinationName);
                SapConnection.singletonConnections.Add(destinationName, retValue);
            }

            return retValue;
        }

        protected SapConnection(String destinationName)
        {
            this.parameters = iniConfiguration.GetParameters(destinationName);
            this.parameters.Remove("IDLE_TIMEOUT");
            this.parameters.Add("IDLE_TIMEOUT", "0");
            this.parameters.Remove("IDLE_CHECK_TIME");
            this.parameters.Add("IDLE_CHECK_TIME", "1");

            this.destination = RfcDestinationManager.GetDestination(destinationName);
            this.customDestination = this.destination.CreateCustomDestination();
            this.customDestination.UseSAPGui = "2";
            this.destinationMonitors = RfcDestinationMonitor.GetMonitorsForCustomDestinations(destinationName);
        }

        public RfcCustomDestination GetDestination
        {
            get
            {
                return this.customDestination;
            }
        }
    }

    public abstract class SapFunctionBase
    {
        protected RfcCustomDestination customDestination;

        protected abstract Boolean Function();

        public static event EventHandler BeforeRfcFunctionCall;
        public static event EventHandler AfterRfcFunctionCall;

        public SapFunctionBase(RfcCustomDestination customDestination)
        {
            this.customDestination = customDestination;
        }

        public Boolean InvokeFunction(String client, String username, String password, String language)
        {
            Boolean retValue = false;

            try
            {
                this.customDestination.Client = client;
                this.customDestination.User = username;
                this.customDestination.Password = password;
                this.customDestination.Language = language;
                                
                if (BeforeRfcFunctionCall != null) BeforeRfcFunctionCall(this, new EventArgs());
                //RfcSessionManager.BeginContext(SapConnection.customDestination);
                retValue = this.Function();

            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
                retValue = false;
            }
            finally
            {
                //RfcSessionManager.EndContext(SapConnection.customDestination);
                if (AfterRfcFunctionCall != null) AfterRfcFunctionCall(this, new EventArgs());
            }

            return retValue;
        }
    }

    // not used today
    public class SapFunctionGetBom : SapFunctionBase
    {
        public SapFunctionGetBom(RfcCustomDestination customDestination)
            : base(customDestination) { }

        override protected Boolean Function()
        {
            IRfcFunction rfcFunction = base.customDestination.Repository.CreateFunction("Z_CS_RFC_IMPORT_BOM");
            rfcFunction.Invoke(base.customDestination);

            IRfcTable table = (IRfcTable)rfcFunction.GetTable("E_BOM_POSITIONS");

            foreach (IRfcStructure Structure in table)
            {
                foreach (IRfcField Field in Structure)
                {
                    Object value = Field.GetValue();
                }
            }

            return true;
        }
    }

    public class SapFunctionSendBom : SapFunctionBase
    {
        private Object[,] bomArray;
        private String matnr;

        public SapFunctionSendBom(RfcCustomDestination customDestination, Object[,] excelBom, String matnr)
            : base(customDestination)
        {
            this.bomArray = excelBom;
            this.matnr = matnr;
        }

        override protected Boolean Function()
        {
            if (this.bomArray == null)
            {
                MessageBox.Show("Excel BOM wasn't read before");
                return false;
            }

            if (this.bomArray.GetLength(0) > 1000)
            {
                MessageBox.Show("More than 1000 Positions");
                return false;
            }

            IRfcFunction rfcFunction = base.customDestination.Repository.CreateFunction("Z_CS_RFC_TRANSFER_BOM");
            RfcStructureMetadata structureMeta = base.customDestination.Repository.GetStructureMetadata("ZCS_S_RFC_STPO");

            rfcFunction.SetValue("I_MATNR", this.matnr);            

            IRfcTable table = (IRfcTable)rfcFunction.GetTable("I_RFC_STPO");

            // Index über Zeilen
            for (int i = 1; i <= this.bomArray.GetLength(0); i++)
            {
                IRfcStructure structure = structureMeta.CreateStructure();

                structure.SetValue("ITEM_NO", CastToString(bomArray[i, 1]));
                structure.SetValue("ITEM_CATEG", CastToString(bomArray[i, 2]));
                structure.SetValue("COMPONENT", CastToString(bomArray[i, 3]));
                structure.SetValue("COMP_QTY", CastToString(bomArray[i, 5]));
                structure.SetValue("COMP_UNIT", CastToString(bomArray[i, 6]));

                table.Append(structure);
            }

            rfcFunction.Invoke(base.customDestination);
            return true;
        }

        private static String CastToString(Object obj)
        {
            if (obj != null)
            {
                return obj.ToString();
            }
            else
            {
                return "";
            }
        }
    }
}